<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App\API;

class ItemController extends \App\Controller
{
    protected $id = null;

    public function __construct($type = null, $mode = null, $id = null)
    {
        $this->id = $id;
        parent::__construct($type, $mode);
    }

    public function process()
    {
        try {
            switch ($this->mode) {
                case 'get':
                    $this->get();
                    break;
                default:
                    $this->json_error('不正なパスです。', null, 404);
                    break;
            }
        } catch (\App\CartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $this->json_error($e->getMessage());
        } catch (\Exception $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $this->json_error($e->getMessage(), null, 500);
        }
    }

    public function json_result($result)
    {
        $this->json($result);
    }

    public function json_success($result = null)
    {
        $this->json(array(
            'success' => true,
            'error' => null,
            'result' => $result,
        ));
    }

    public function json_error($error, $result = null, $code = 400)
    {
        $this->json(array(
            'success' => false,
            'error' => $error,
            'result' => $result,
        ), $code);
    }

    public function render($template, $data, $code = 200)
    {
        if($code == 200) {
            $this->json_success($data);
        }
        else {
            $error = 'エラーが発生しました。';
            if($template == '_maintenance.html') {
                $error = 'メンテナンス中です。';
            }
            $this->json_error($error, $data, $code);
        }
    }

    public function get()
    {
        if(is_null($this->id)) {
            return $this->json_error('商品IDが指定さあれていません。', null, 403);
        } 
        $item = \App::item()->find($this->id);
        if ($item === false) {
            return $this->json_error('商品が見つかりません。', null, 404);
        }
        return $this->json_success($item->getProperties());
    }

}
