<?php

namespace App;

class CharCodeChecker
{
    private static $ASCII_REGEX = "[\x20-\x7E]";

    public static function isASCII($string)
    {
        return preg_match('/^' . self::$ASCII_REGEX . '+$/', $string) ? true : false;
    }

    public static function isJISX0208($string)
    {
        $len = mb_strlen($string);
        for($i=0; $i<$len; $i++) {
            $ch = mb_substr($string, $i, 1);
            if(self::isASCII($ch)) {
                continue;
            }
            if(!self::isJISX0208Char($ch)) {
                return false;
            }                        
        }
        return true;
    }

    public static function isJISX0208Char($ch)
    {
        $bytes = array_values(unpack('C*', mb_convert_encoding($ch, 'jis')));
        if(count($bytes) != 8) {
            return false;
        }

        $row = $bytes[3] - 0x20;
        $cell = $bytes[4] - 0x20;
    
        switch($row)
        {
            case 1:
                if($cell >= 1 && $cell <= 94) {
                    return true;
                }
                break;
            case 2:
                if(($cell >= 1 && $cell <= 14) 
                    || ($cell >= 26 && $cell <= 33) 
                    || ($cell >= 42 && $cell <= 48)
                    || ($cell >= 60 && $cell <= 74)
                    || ($cell >= 82 && $cell <= 89)
                    || ($cell === 94)
                ) {
                    return true;
                }
                break;
            case 3:
                if(($cell >= 16 && $cell <= 25)
                    || ($cell >= 33 && $cell <= 58)
                    || ($cell >= 65 && $cell <= 90)
                ) {
                    return true;
                }
                break;
            case 4:
                if($cell >= 1 && $cell <= 83) {
                    return true;
                }
                break;
            case 5:
                if($cell >= 1 && $cell <= 86) {
                    return true;
                }
                break;
            case 6:
                if(($cell >= 1 && $cell <= 24)
                    || ($cell >= 33 && $cell <= 56)
                ) {
                    return true;
                }
                break;
            case 7:
                if(($cell >= 1 && $cell <= 33)
                    || ($cell >= 49 && $cell <= 81)
                ) {
                    return true;
                }
                break;
            case 8:
                if($cell >= 1 && $cell <= 32) {
                    return true;
                }
                break;
            default:
                if(($row >= 16 && $row <= 46)
                    || ($row >= 48 && $row <= 83)
                ) {
                    if($cell >= 1 && $cell <= 94) {
                        return true;
                    }
                }
                else if($row === 47) {
                    if($cell >= 1 && $cell <= 51) {
                        return true;
                    }
                }
                else if($row === 84) {
                    if($cell >= 1 && $cell <= 6) {
                        return true;
                    }
                }
                break;
        }

        return false;
    }

    public static function filterJISX0208($string)
    {
        $output = '';
        $len = mb_strlen($string);
        for($i=0; $i<$len; $i++) {
            $ch = mb_substr($string, $i, 1);
            if(self::isASCII($ch) || self::isJISX0208Char($ch)) {
                $output .= $ch;
            }                        
        }
        return $output;
    }
}