<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;


class Postage
{
    protected $table;
    protected $freePostagePrice;

    public function __construct($table, $freePostagePrice)
    {
        if (!isset($table) || !is_array($table)) {
            throw new \UnexpectedValueException('不正な送料データです。');
        }
        if (!isset($freePostagePrice)) {
            throw new \UnexpectedValueException('送料無料価格が指定されていません。');
        }
        $this->table = $table;
        $this->freePostagePrice = $freePostagePrice;
    }

    public function setTable($table)
    {
        $this->table = $table;
    }

    public function isFreePostage(Cart $cart, Slip $slip)
    {
        if ($this->freePostagePrice !== false) {
            if ($slip->get('subtotal') >= $this->freePostagePrice) {
                return true;
            }
        }
        return false;
    }

    public function calculate(Cart $cart, Slip $slip)
    {
        if (!isset($cart)) {
            throw new \UnexpectedValueException('Missing argument - cart');
        }
        if (!isset($slip)) {
            throw new \UnexpectedValueException('Missing argument - slip');
        }

        $prefId = $slip->getDestinationPrefecture();
        if (is_string($prefId)) {
            $prefId = intval($prefId);
        }
        if (!isset($prefId) || $prefId < 1 || $prefId > 47) {
            return 0;
        }
        $postage = $this->postage($prefId);
        if ($this->isFreePostage($cart, $slip)) {
            $postage = 0;
        }
        return $postage;
    }

    public function postage($prefId)
    {
        $table = $this->table();
        return $table[$prefId];
    }

    public function table()
    {
        return $this->table;
    }
}
