<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class Route
{
    public static function register()
    {
        $route = [
            [
                '/',
                function () {
                    \App::cartController()->start('show');
                },
                false,
            ],
            [
                '/index.php',
                function () {
                    \App::cartController()->start('show');
                },
                false,
            ],
            [
                '/@type:cart|checkout(/@mode)',
                function ($type, $mode) {
                    $controller = call_user_func(array('\App', $type . 'Controller'));
                    $controller->start($mode);
                },
                false,
            ],
            [
                '/tpc(/@mode)',
                function ($mode) {
                    $controller = new \App\TPCController();
                    $controller->start($mode);
                },
                false,
            ],
            [
                '/api/cart(/@mode)',
                function ($mode) {
                    $controller = new \App\API\CartController();
                    $controller->start($mode);
                },
                false,
            ],
            [
                '/api/item/@mode(/@id)',
                function ($mode, $id) {
                    $controller = new \App\API\ItemController('item', $mode, $id);
                    $controller->start($mode);
                },
                false,
            ],
            [
                '/api/error',
                function () {
                    return 1 / 0;
                },
                false,
            ]
        ];
        if(\App::config()->isPreviewMode()) {
            $route[] = array(
                '/preview/@type/@mode',
                function ($type, $mode) {
                    $controller = new PreviewController($type, $mode);
                    $controller->start($mode);
                },
                false,
            );
        }
        foreach ($route as $params) {
            \App::route($params[0], $params[1], isset($params[2]) ? $params[2] : false);
        }
    }
}
