<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class Session
{
    private $started = false;
    private $data = array();

    public function __construct()
    {
        if (PHP_SAPI === 'cli') {
            $_SESSION = array();
        }
    }

    public function start()
    {
        if ($this->started === false) {
            $current_cookie_params = session_get_cookie_params();
            $is_ssl = Util::is_ssl();
            $httponly = true;
            // NOTE: SameSite=None 対応
            $allowedOrigin = \App::config()->getAllowedOrigin();
            if(version_compare(PHP_VERSION, '7.3.0', '<')) {
                session_set_cookie_params(
                    $current_cookie_params['lifetime'],
                    $allowedOrigin ? '/; SameSite=None' : '/',
                    $current_cookie_params['domain'],
                    $is_ssl, // secure
                    $httponly
                );
            }
            else {
                session_set_cookie_params(array(
                    'lifetime' => $current_cookie_params['lifetime'],
                    'path' => '/',
                    'domain' => $current_cookie_params['domain'],
                    'secure' => $is_ssl,
                    'httponly' => $httponly,
                    'samesite' => $allowedOrigin ? 'None' : '',
                ));
            }
            session_name('skeletoncart' . \App::config()->get('cart_item_site_id', ''));
            session_start();
            $this->load();
            $this->started = true;
        }
        return true;
    }

    public function load()
    {
        if (isset($_SESSION['SKELETON_CART_DATA'])) {
            $data = $_SESSION['SKELETON_CART_DATA'];
            $this->data = unserialize($data);
        }
    }

    public function save()
    {
        $data = serialize($this->data);
        $_SESSION['SKELETON_CART_DATA'] = $data;
    }

    public function get($key)
    {
        if (!$key) {
            return null;
        }
        if (!isset($this->data[$key])) {
            return null;
        }
        $value = $this->data[$key];
        return $value;
    }

    public function set($key, $value)
    {
        if (!$key) {
            return false;
        }
        $this->data[$key] = $value;
        return true;
    }
}

