<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

use \App\Util;

class Slip
{
    protected static $default = array(
        'id' => null,
        'date' => null,
        'user_name' => null,
        'user_name_sei' => null,
        'user_name_mei' => null,
        'user_name_kana' => null,
        'user_name_sei_kana' => null,
        'user_name_mei_kana' => null,
        'user_post' => null,
        'user_prefecture' => null,
        'user_city' => null,
        'user_address1' => null,
        'user_address2' => null,
        'user_tel' => null,
        'user_fax' => null,
        'user_mail' => null,
        'user_opt1' => null,
        'user_opt2' => null,
        'user_opt3' => null,
        'user_opt4' => null,
        'user_opt5' => null,
        'user_opt6' => null,
        'user_opt7' => null,
        'user_opt8' => null,
        'user_opt9' => null,
        'user_opt10' => null,
        'delivery_flg' => null,
        'delivery_name' => null,
        'delivery_post' => null,
        'delivery_prefecture' => null,
        'delivery_city' => null,
        'delivery_address1' => null,
        'delivery_address2' => null,
        'delivery_tel' => null,
        'delivery_opt1' => null,
        'delivery_opt2' => null,
        'delivery_opt3' => null,
        'delivery_opt4' => null,
        'delivery_opt5' => null,
        'delivery_opt6' => null,
        'delivery_opt7' => null,
        'delivery_opt8' => null,
        'delivery_opt9' => null,
        'delivery_opt10' => null,
        'payment_way' => 0,
        'delivery_date_mon' => null,
        'delivery_date_day' => null,
        'delivery_date' => null,
        'delivery_time_category' => null,
        'weight' => 0,
        'subtotal' => 0,
        'tax' => 0,
        'fee' => 0,
        'postage' => 0,
        'payment' => 0,
        'memo' => null,
    );
    protected static $rules;
    protected static $labels;

    protected $props = array();
    protected $extraProps = array();

    public function __construct($params = null)
    {
        $this->setProperties($params);
    }

    public function setProperties($params)
    {
        if (isset($params)) {
            $params = array_merge($this->getDefaultProperties(), $params);
            $params = $this->mergeDeliveryDate($params);
            $this->props = $this->validateParams($params);
            $this->updatePayment();
        } else {
            $this->props = $this->getDefaultProperties();
        }
        if (!isset($this->props['id'])) {
            $this->props['id'] = date("YmdHis");
        }
        if (!isset($this->props['date'])) {
            $this->props['date'] = date("Y-m-d H:i:s");
        }
    }

    public function mergeDeliveryDate($params)
    {
        // delivery_dateがセットされている場合、そこから_monと_dayをセットする。
        if(!Util::is_empty($params['delivery_date'])) {
            $date = date_create($params['delivery_date']);
            if(!$date) {
                return $params;
            }
            $params['delivery_date_mon'] = $date->format('m');
            $params['delivery_date_day'] = $date->format('d');
        }
        // delivery_dateがセットされていない場合、
        else {
            // _monと_dayがセットされていれば、deliver_dateをセットする。
            if(!Util::is_empty($params['delivery_date_mon'])
                && !Util::is_empty($params['delivery_date_day'])) {
                $today = date_create('today');
                $date = date_create($today->format('Y').'-'.$params['delivery_date_mon'].'-'.$params['delivery_date_day']);
                if(!$date) {
                    return $params;
                }
                // 今日以前の月日の場合は、来年の日付とする。
                if($date < $today) {
                    $date->modify('+1 year');
                }
                $params['delivery_date'] = $date->format('Y-m-d');
            }
        }
        return $params;
    }

    public function validateParams($params)
    {
        if (!self::$rules) {
            self::$rules = array(
                'user_name' => array(),
                'user_name_sei' => array(),
                'user_name_mei' => array(),
                'user_name_kana' => array(),
                'user_name_sei_kana' => array(),
                'user_name_mei_kana' => array(),
                'user_post' => array(),
                'user_prefecture' => array(),
                'user_city' => array(),
                'user_address1' => array(),
                'user_address2' => array(),
                'user_tel' => array(),
                'user_fax' => array(),
                'user_mail' => array(
                    'EMAIL' => "以下のようなメールアドレスはご利用いただけません。\n ・@ の直前にピリオド（.）がある\n ・@ より前に連続したピリオドがある\n ・@ が複数入っている",
                ),
                'user_opt1' => array(),
                'user_opt2' => array(),
                'user_opt3' => array(),
                'user_opt4' => array(),
                'user_opt5' => array(),
                'user_opt6' => array(),
                'user_opt7' => array(),
                'user_opt8' => array(),
                'user_opt9' => array(),
                'user_opt10' => array(),
                'delivery_flg' => array(),
                'delivery_name' => array(),
                'delivery_post' => array(),
                'delivery_prefecture' => array(),
                'delivery_city' => array(),
                'delivery_address1' => array(),
                'delivery_address2' => array(),
                'delivery_tel' => array(),
                'delivery_opt1' => array(),
                'delivery_opt2' => array(),
                'delivery_opt3' => array(),
                'delivery_opt4' => array(),
                'delivery_opt5' => array(),
                'delivery_opt6' => array(),
                'delivery_opt7' => array(),
                'delivery_opt8' => array(),
                'delivery_opt9' => array(),
                'delivery_opt10' => array(),
                'payment_way' => array(),
                'delivery_date_mon' => array(),
                'delivery_date_day' => array(),
                'delivery_date' => array(),
                'delivery_time_category' => array(),
                'weight' => array(),
                'subtotal' => array(),
                'tax' => array(),
                'fee' => array(),
                'postage' => array(),
                'payment' => array(),
                'memo' => array(),
            );
            $this->buildValidatorRules();
        }

        // Filter input values
        $params = $this->filterParams($params);

        // Validate input values
        $validator = \App::validator(true);
        $validator->setRules(self::$rules);
        \App::runCallbacks('addSlipValidationRule', $validator);
        if (!$validator->validate($params)) {
            $errors = $validator->getErrors();
            throw new ValidationException('パラメーラの値が不正です。', $errors);
        }
        return $params;
    }

    public function filterParams($params)
    {
        $filteredParams = array();
        foreach ($params as $k => $v) {
            if (array_key_exists($k, self::$default)) {
                $filteredParams[$k] = $v;
            }
        }
        array_walk($filteredParams, function (&$value, $key) {
            if (isset($value)) {
                $value = Util::zen2han($value);
            }
        });

        return $filteredParams;
    }

    public function isValid()
    {
        $props = $this->props;
        try {
            $this->validateParams($props);
        } catch (ValidationException $e) {
            return false;
        }
        return true;
    }

    public function buildValidatorRules()
    {
        foreach (array_keys(self::$rules) as $name) {
            $rule = &self::$rules[$name];
            $inputCheck = \App::config()->inputCheck('slip_' . $name);
            if ($inputCheck === false) {
                // デフォルト
            } else {
                if ($name === 'delivery_flg') {
                    continue;
                }
                $label = $this->getParamLabel($name);
                $required = isset($inputCheck[0]) ? intval($inputCheck[0]) : 0;
                if ($required) {
                    $pattern = '/^delivery_(?:name|post|prefecture|city|address1|address2|tel|opt\d+)$/';
                    $required_rule = array();
                    if (preg_match($pattern, $name)) {
                        $required_rule['REQUIRED_IF'] = array(
                            'args' => array('key' => 'delivery_flg'),
                            'message' => $label . 'が記入されていないか、内容に不備があります。',
                        );
                    } else {
                        $required_rule['REQUIRED'] = $label . 'が記入されていないか、内容に不備があります。';
                    }
                    // REQUIRED ルールを先頭にする
                    $rule = array_merge($required_rule, $rule);
                }
                $length = isset($inputCheck[1]) ? intval($inputCheck[1]) : false;
                if ($length !== false) {
                    $rule['LENGTH'] = array(
                        'message' => $label . 'の文字数が指定文字数を超えています。',
                        'args' => array('max' => $length),
                    );
                }
                if ($name === 'payment_way') {
                    $rule['CUSTOM_PAYMENT_WAY'] = array(
                        'func' => array($this, 'validatePaymentWay'),
                        'message' => 'ご注文者様と別の住所へ発送する場合、「代金引換」はご利用いただけません。',
                    );
                }
                if ($name === 'delivery_date') {
                    $rule['DATE_FMT'] = array(
                        'message' => $label . 'は正しい日付を入力してください。',
                    );
                }
            }
        }
    }

    public function validatePaymentWay($key, $value, $params, $args)
    {
        if ($value == 0) {
            return false;
        }
        $type = \App::config()->getPaymentWayType($value);
        $delivery = false;
        if (array_key_exists('delivery_flg', $params)) {
            $delivery = $params['delivery_flg'];
        }
        if ($type == 1 && $delivery) {
            return false;
        }
        return true;
    }

    public function getParamLabel($name)
    {
        if (!self::$labels) {
            self::$labels = array(
                'id' => '注文ID',
                'date' => '注文日',
                'user_name' => '注文者名',
                'user_name_sei' => '注文者名(姓)',
                'user_name_mei' => '注文者名(名)',
                'user_name_kana' => '注文者名(フリガナ)',
                'user_name_sei_kana' => '注文者名(姓)(フリガナ)',
                'user_name_mei_kana' => '注文者名(名)(フリガナ)',
                'user_post' => '注文者住所(郵便番号)',
                'user_prefecture' => '注文者住所(都道府県)',
                'user_city' => '注文者住所(市区町村)',
                'user_address1' => '注文者住所(番地等)',
                'user_address2' => '注文者住所(その他)',
                'user_tel' => '注文者電話番号',
                'user_fax' => '注文者FAX番号',
                'user_mail' => '注文者メールアドレス',
                'user_opt1' => \App::config()->slip_user_opt1_label,
                'user_opt2' => \App::config()->slip_user_opt2_label,
                'user_opt3' => \App::config()->slip_user_opt3_label,
                'user_opt4' => \App::config()->slip_user_opt4_label,
                'user_opt5' => \App::config()->slip_user_opt5_label,
                'user_opt6' => \App::config()->slip_user_opt6_label,
                'user_opt7' => \App::config()->slip_user_opt7_label,
                'user_opt8' => \App::config()->slip_user_opt8_label,
                'user_opt9' => \App::config()->slip_user_opt9_label,
                'user_opt10' => \App::config()->slip_user_opt10_label,
                'delivery_flg' => '配送先使用フラグ',
                'delivery_name' => '配送先名',
                'delivery_post' => '配送先住所(郵便番号)',
                'delivery_prefecture' => '配送先住所(都道府県)',
                'delivery_city' => '配送先住所(市区町村)',
                'delivery_address1' => '配送先住所(番地等)',
                'delivery_address2' => '配送先住所(その他)',
                'delivery_tel' => '配送先電話番号',
                'delivery_opt1' => \App::config()->slip_delivery_opt1_label,
                'delivery_opt2' => \App::config()->slip_delivery_opt2_label,
                'delivery_opt3' => \App::config()->slip_delivery_opt3_label,
                'delivery_opt4' => \App::config()->slip_delivery_opt4_label,
                'delivery_opt5' => \App::config()->slip_delivery_opt5_label,
                'delivery_opt6' => \App::config()->slip_delivery_opt6_label,
                'delivery_opt7' => \App::config()->slip_delivery_opt7_label,
                'delivery_opt8' => \App::config()->slip_delivery_opt8_label,
                'delivery_opt9' => \App::config()->slip_delivery_opt9_label,
                'delivery_opt10' => \App::config()->slip_delivery_opt10_label,
                'payment_way' => '支払方法',
                'delivery_date_mon' => '配送希望日時(月)',
                'delivery_date_day' => '配送希望日時(日)',
                'delivery_date' => '配送指定日',
                'delivery_time_category' => '配送希望時間指定区分',
                'memo' => '備考',
            );
        }
        return array_key_exists($name, self::$labels) ? self::$labels[$name] : null;
    }

    public function getCompatibleProperties()
    {
        $props = array();
        foreach ($this->props as $k => $v) {
            $props['slip_' . $k] = $v;
        }
        $props['slip_payment_way_name'] = \App::config()->getPaymentWayName($props['slip_payment_way']);

        $extraProps = $this->getCompatibleExtraProperties();
        $props = array_merge($props, $extraProps);

        return $props;
    }

    public function getDefaultProperties()
    {
        $defaults = array(
            'payment_way' => \App::config()->getDefaultPaymentWay(),
        );
        $props = array_merge(self::$default, $defaults);
        return $props;
    }

    public function load($session)
    {
        if (!isset($session)) {
            throw new \UnexpectedValueException('Missing argument - session');
        }
        $slip = $session->get('slip');
        if ($slip && $slip instanceof Slip) {
            $this->setProperties($slip->props);
            // NOTE: extraPropsはConstructorでは設定されない
            if (is_array($slip->extraProps)) {
                $this->setExtraProperties($slip->extraProps);
            }
        }
    }

    public function save($session)
    {
        if (!isset($session)) {
            throw new \UnexpectedValueException('Missing argument - session');
        }
        $this->updatePayment();

        $session->set('slip', $this);
        $session->save();
    }

    public function updatePayment()
    {
        \App::runCallbacks('preUpdateSlip', $this);

        $config = \App::config();
        $this->props['tax'] = intval(floor($this->props['subtotal'] * $config->getTaxRate()));
        $this->props['fee'] = $config->getPaymentWayFee($this->props['payment_way']);
        $this->props['postage'] = $this->getPostage()->calculate($this->getCart(), $this);
        $this->props['payment'] = $this->props['subtotal']
                                  + $this->props['tax']
                                  + $this->props['fee']
                                  + $this->props['postage'];

        \App::runCallbacks('postUpdateSlip', $this);
    }

    public function weight()
    {
        return $this->props['weight'];
    }

    public function setWeight($weight)
    {
        if (!is_int($weight)) {
            throw new \UnexpectedValueException('Weight is not an integrt');
        }
        $this->props['weight'] = $weight;
    }

    public function subTotal()
    {
        return $this->props['subtotal'];
    }

    public function setSubTotal($subTotal)
    {
        if (!is_int($subTotal)) {
            throw new \UnexpectedValueException('SubTotal is not an integer');
        }
        $this->props['subtotal'] = $subTotal;
    }

    public function tax()
    {
        return $this->props['tax'];
    }

    public function setTax($tax)
    {
        if (!is_numeric($tax)) {
            throw new \UnexpectedValueException('Tax is not an numeric');
        }
        $this->props['tax'] = $tax;
    }

    public function fee()
    {
        return $this->props['fee'];
    }

    public function setFee($fee)
    {
        if (!is_numeric($fee)) {
            throw new \UnexpectedValueException('Fee is not an numeric');
        }
        $this->props['fee'] = $fee;
    }

    public function postage()
    {
        return $this->props['postage'];
    }

    public function setPostage($postage)
    {
        if (!is_numeric($postage)) {
            throw new \UnexpectedValueException('Postage is not an numeric');
        }
        $this->props['postage'] = $postage;
    }

    public function payment()
    {
        return $this->props['payment'];
    }

    public function setPayment($payment)
    {
        if (!is_numeric($payment)) {
            throw new \UnexpectedValueException('Payment is not an numeric');
        }
        $this->props['payment'] = $payment;
    }

    public function getPostage()
    {
        return \App::postage();
    }

    public function getCart()
    {
        return \App::cart();
    }

    public function getDestinationPrefecture()
    {
        $prefId = $this->props['delivery_flg']
            ? $this->props['delivery_prefecture'] : $this->props['user_prefecture'];
        return $prefId;
    }

    public function get($name, $default = null)
    {
        if (array_key_exists($name, $this->props)) {
            return $this->props[$name];
        }
        if (preg_match('/^((?:user|delivery)_prefecture)_name$/', $name, $matches)) {
            return Util::prefCode2name($this->props[$matches[1]]);
        }
        return $default;
    }

    public function setExtraProperty($name, $values)
    {
        if (!is_string($name)) {
            throw new \InvalidArgumentException('Invalid argument - name');
        }
        if (array_key_exists($name, self::$default)) {
            throw new \InvalidArgumentException('Duplicate property');
        }
        if (!is_array($values) || count($values) == 0) {
            throw new \InvalidArgumentException('Invalid argumant - prop');
        }
        $this->extraProps[$name] = $values;
    }

    public function removeExtraProperty($name)
    {
        if (!is_string($name)) {
            throw new \InvalidArgumentException('Invalid argument - name');
        }
        if (array_key_exists($name, $this->extraProps)) {
            unset($this->extraProps[$name]);
        }
    }

    public function getExtraProperty($name)
    {
        if (!is_string($name)) {
            throw new \InvalidArgumentException('Invalid argument - name');
        }
        if (array_key_exists($name, $this->extraProps)) {
            return $this->extraProps[$name];
        }
        return false;
    }

    public function getCompatibleExtraProperties()
    {
        $props = array();
        foreach ($this->extraProps as $name => $values) {
            foreach ($values as $k => $v) {
                $props["slip_{$name}_{$k}"] = $v;
            }
        }
        return $props;
    }

    public function getExtraProperties()
    {
        return $this->extraProps;
    }

    public function setExtraProperties($extraProps)
    {
        if (!is_array($extraProps)) {
            throw new \InvalidArgumentException('Invalid argument - extraProps');
        }
        $this->extraProps = $extraProps;
    }

    public function getProperties()
    {
        $props = array();
        foreach ($this->props as $k => $v) {
            $props[$k] = $v;
        }
        $props['payment_way_name'] = \App::config()->getPaymentWayName($props['payment_way']);

        $extraProps = array();
        foreach ($this->extraProps as $name => $values) {
            foreach ($values as $k => $v) {
                $extraProps["{$name}_{$k}"] = $v;
            }
        }
        $props = array_merge($props, $extraProps);

        return $props;
    }
}
