<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class TPCController extends Controller
{
    public function __construct($mode = null)
    {
        parent::__construct('tpc', $mode);
        \App::addCallback('preRender', array($this, 'preRender'));
    }

    public function process()
    {
        try {
            if ($this->mode === 'start') {
                $this->checkStart();
            }
            else if($this->mode === 'complete') {
                $this->checkComplete();
            }
            else {
                $this->checkStart();
            }
        }
        catch (\Exception $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            $this->render('_error.html', $data);
        }
    }
    
    public function getTemplateParams()
    {
        $data = array(
            'error_id' => null,
            'error_message' => null,
            'app_script' => \App::appScriptName(),
            'app_dir' => \App::appScriptDir(),
            'base_url' => $this->base_url(),
        );
        return $data;
    }

    public function checkStart()
    {
        $data = $this->getTemplateParams();
        $this->render('_tpc_start.html', $data);
    }

    public function checkComplete()
    {
        $data = $this->getTemplateParams();
        $this->render('_tpc_complete.html', $data);
    }

    public function preRender($template, $data)
    {
        if($template === '_tpc_start.html' || $template === '_tpc_complete.html') {
            header_remove('X-Frame-Options');
        }
    }
}

