<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class Util
{
    public static function is_empty($string)
    {
        return !(isset($string) && strlen($string) > 0);
    }

    public static function underscore($string)
    {
        return ltrim(strtolower(preg_replace('/[A-Z]/', '_\0', $string)), '_');
    }

    public static function camelize($string)
    {
        return lcfirst(strtr(ucwords(strtr($string, array('_' => ' '))), array(' ' => '')));
    }

    public static function zen2han($string)
    {
        return mb_convert_kana($string, "Ka");
    }

    public static function is_ssl()
    {
        if (isset($_SERVER['HTTPS']) === true) {
            // Apache
            return ($_SERVER['HTTPS'] === 'on' or $_SERVER['HTTPS'] === '1');
        } elseif (isset($_SERVER['SSL']) === true) {
            // IIS
            return ($_SERVER['SSL'] === 'on');
        } elseif (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) === true) {
            // Reverse proxy
            return (strtolower($_SERVER['HTTP_X_FORWARDED_PROTO']) === 'https');
        } elseif (isset($_SERVER['HTTP_X_FORWARDED_PORT']) === true) {
            // Reverse proxy
            return ($_SERVER['HTTP_X_FORWARDED_PORT'] === '443');
        } elseif (isset($_SERVER['SERVER_PORT']) === true) {
            return ($_SERVER['SERVER_PORT'] === '443');
        }
        return false;
    }

    public static function prefCode2name($code)
    {
        $pref_codes = array(
            "1" => "北海道",
            "2" => "青森県",
            "3" => "岩手県",
            "4" => "宮城県",
            "5" => "秋田県",
            "6" => "山形県",
            "7" => "福島県",
            "8" => "茨城県",
            "9" => "栃木県",
            "10" => "群馬県",
            "11" => "埼玉県",
            "12" => "千葉県",
            "13" => "東京都",
            "14" => "神奈川県",
            "15" => "新潟県",
            "16" => "富山県",
            "17" => "石川県",
            "18" => "福井県",
            "19" => "山梨県",
            "20" => "長野県",
            "21" => "岐阜県",
            "22" => "静岡県",
            "23" => "愛知県",
            "24" => "三重県",
            "25" => "滋賀県",
            "26" => "京都府",
            "27" => "大阪府",
            "28" => "兵庫県",
            "29" => "奈良県",
            "30" => "和歌山県",
            "31" => "鳥取県",
            "32" => "島根県",
            "33" => "岡山県",
            "34" => "広島県",
            "35" => "山口県",
            "36" => "徳島県",
            "37" => "香川県",
            "38" => "愛媛県",
            "39" => "高知県",
            "40" => "福岡県",
            "41" => "佐賀県",
            "42" => "長崎県",
            "43" => "熊本県",
            "44" => "大分県",
            "45" => "宮崎県",
            "46" => "鹿児島県",
            "47" => "沖縄県"
        );
        if(array_key_exists($code, $pref_codes)) {
            return $pref_codes[$code];
        }
        return '';
    }

    public static function sanitize_url($url)
    {
        # Check valid chars -- rfc3986
        if(preg_match('/[^;\/?:@&=+\$,A-Za-z0-9\-_.!~*\'()%#]/', $url)) {
            return '';
        }

        # Check valid scheme
        if(preg_match('/^([A-Za-z][A-Za-z0-9+\-.]*):/', $url, $matches)) {
            $scheme = strtolower($matches[1]);
            $allow = 0;
            if($scheme === 'http'
                || $scheme === 'https'
                || $scheme === 'mailto') {
                $allow = 1;
            }
            if(!$allow) {
                return '';
            }
        }

        /* NOTE: commented out the followings, because normally escaped with HTML by default in SkeletonCart view. */
        /*
        # escape
        $url = preg_replace('/&/', '&amp;', $url);
        $url = preg_replace('/\'/', '&#39;', $url);
         */

        return $url;
    }

    public static function make_url($type, $mode='', $include_base_dir=true)
    {
        if(empty($type)) {
            $type = 'cart';
        }
        $base_path = $include_base_dir ? \App::appBasedir() : '';
        $paths = explode('/', $base_path);

        $type_path = '';
        if($type === 'cart') {
            $type_path = \App::config()->getCartPath();
        }
        else if($type === 'checkout') {
            $type_path = \App::config()->getCheckoutPath();
        }
        else {
            // TODO: Error?
            $type_path = $type;
        }
        $paths = array_merge($paths, explode('/', $type_path));
        $paths[] = $mode;

        $paths = array_reduce($paths,
            function($carry, $item) {
                if(isset($item) && strlen($item) > 0) {
                    array_push($carry, $item);
                }
                return $carry;
            },
            array()
        );

        $path = '/' . implode('/', $paths);
        $path = self::sanitize_url($path);
        return $path;
    }

    public static function get_form_type($form)
    {
        $path = '';
        if($form === 'cart') {
            $path = \App::config()->getCartPath();
        }
        else if($form === 'checkout') {
            $path = \App::config()->getCheckoutPath();
        }
        $types = array_filter(explode('/', $path),
            function($item) {
                return strlen($item);
            }
        );
        $type = end($types); reset($types);
        return $type;
    }
}

